﻿Imports System.ComponentModel
Imports System.Runtime.CompilerServices
Namespace Methods
    <Extension()> _
    Public Module Extensions

        ''' <summary>
        ''' Turns A Byte array into a String assuming that it is UTF-8 encoded.
        ''' </summary>
        ''' <param name="b">Byte array to convert. placed automaticly by extension system.</param>
        ''' <returns>String</returns>
        ''' <remarks>shortcut to System.Text.UnicodeEncoding.UTF8.GetString</remarks>
        <Extension()> _
        Public Function GetString(ByVal b As Byte()) As String
            Return System.Text.UnicodeEncoding.UTF8.GetString(b)
        End Function

        ''' <summary>
        ''' Turns A String into an UTF-8 encoded Byte array.
        ''' </summary>
        ''' <param name="s">String to convert. placed automaticly by extension system.</param>
        ''' <returns>Byte()</returns>
        ''' <remarks>shortcut to System.Text.UnicodeEncoding.UTF8.GetBytes</remarks>
        <Extension()> _
        Public Function GetBytes(ByVal s As String) As Byte()
            Return System.Text.UnicodeEncoding.UTF8.GetBytes(s)
        End Function

        ''' <summary>
        ''' Places data from a certin coloumn of a DataTable into an array.
        ''' </summary>
        ''' <param name="table">the DataTable to query. placed automaticly by extension system.</param>
        ''' <param name="ColoumnIndex">The zero-based index of the target coloumn.</param>
        ''' <returns>Object()</returns>
        ''' <remarks>Linq query that returns a certain coloumn's data.</remarks>
        <Extension()> _
        Public Function GetColoumnData(ByVal table As DataTable, ByVal ColoumnIndex As Integer) As Object()
            Return (From r In table Select r.Item(ColoumnIndex)).ToArray
        End Function

        ''' <summary>
        ''' Gets the day of year for a year that starts at an abnormal date. i.e. September 1st.
        ''' </summary>
        ''' <param name="QuestionDate">The date you wish to find the day of year for. placed automaticly by extension system.</param>
        ''' <param name="YearStartDate">The starting date for the financial year.
        ''' NOTE: year is ignored. Only month and day are used.</param>
        ''' <returns>Integer</returns>
        ''' <remarks>errors may accur when the year paces a leap year.
        ''' however the error should only be a matter of one or two days.</remarks>
        <Extension()> _
       Public Function GetDayOfFinancalYear(ByVal QuestionDate As Date, ByVal YearStartDate As Date) As Integer
            YearStartDate = New Date(Now.Year, YearStartDate.Month, YearStartDate.Day)
            If QuestionDate.DayOfYear - YearStartDate.DayOfYear >= 0 Then
                Return QuestionDate.DayOfYear - YearStartDate.DayOfYear + 1
            Else
                Return QuestionDate.DayOfYear - YearStartDate.DayOfYear + 365
            End If
        End Function

    End Module
End Namespace