﻿Imports System.Drawing.Drawing2D
Imports System.ComponentModel
Namespace GradientControls
    <Description("The base class for all gradient controls can be styled to mimck other controls.")> _
    Public Class gradientBase
        Inherits Control
        Private border_size As Integer
        Private border_color As Color
        Private s As New styles.style
        Private _borderStyle As New styles.style
        Private _UseGradientBorderStyle As Boolean = False
        <Browsable(False), RefreshProperties(RefreshProperties.All)> _
        Public Property GradientStyle() As styles.style
            Get
                Return s
            End Get
            Set(ByVal value As styles.style)
                s = value
                Refresh()
            End Set
        End Property

        Public Property borderStyle() As styles.style
            Get
                Return _borderStyle
            End Get
            Set(ByVal value As styles.style)
                _borderStyle = value
                Me.Refresh()
            End Set
        End Property

        Public Property UseGradientBorderStyle() As Boolean
            Get
                Return _UseGradientBorderStyle
            End Get
            Set(ByVal value As Boolean)
                _UseGradientBorderStyle = value
            End Set
        End Property

        <System.ComponentModel.Description("The array of positions as a decimal, E.G. 0.75, that corresponds to the gradient color of the same index in the multicolor gradient."), _
        System.ComponentModel.Category("Appearance"), MergableProperty(True)> _
        Public Property GradientPositions() As Collections.Generic.List(Of Single)
            Get
                Return s.GradientPositions
            End Get
            Set(ByVal value As Collections.Generic.List(Of Single))
                s.GradientPositions = value
                Me.Refresh()
            End Set
        End Property

        <System.ComponentModel.Description("A generic list of colors that make up the multicolor gradient."), _
        System.ComponentModel.Category("Appearance"), MergableProperty(True)> _
        Public Property GradientColorArray() As Collections.Generic.List(Of Color)
            Get
                Return s.GradientColorArray
            End Get
            Set(ByVal value As Collections.Generic.List(Of Color))
                s.GradientColorArray = value
                Me.Refresh()
            End Set
        End Property

        <System.ComponentModel.Description("The angle to rotate the gradient."), _
        System.ComponentModel.DefaultValue(90), _
        System.ComponentModel.Category("Appearance"), MergableProperty(True)> _
        Public Property GradientAngle() As Integer
            Get
                Return s.GradientAngle
            End Get
            Set(ByVal value As Integer)
                s.GradientAngle = value
                Me.Refresh()
            End Set
        End Property

        <System.ComponentModel.Description("The size of the controls border."), _
        System.ComponentModel.Category("Appearance")> _
        Public Property BorderSize() As Integer
            Get
                Return border_size
            End Get
            Set(ByVal value As Integer)
                border_size = value
                Me.Refresh()
            End Set
        End Property

        <System.ComponentModel.Description("The color of the controls border."), _
        System.ComponentModel.Category("Appearance")> _
        Public Property BorderColor() As Color
            Get
                Return border_color
            End Get
            Set(ByVal value As Color)
                border_color = value
                Me.Refresh()
            End Set
        End Property

        Private x1 As StringAlignment
        Private y1 As StringAlignment

        <System.ComponentModel.Description("The distance of the text from the left."), _
        System.ComponentModel.Category("Appearance")> _
        Public Property TextAlinment_horizantal() As StringAlignment
            Get
                Return x1
            End Get
            Set(ByVal value As StringAlignment)
                x1 = value
                Me.Refresh()
            End Set
        End Property

        <System.ComponentModel.Description("The distance of the text from the top."), _
        System.ComponentModel.Category("Appearance")> _
        Public Property TextAlinment_vertical() As StringAlignment
            Get
                Return y1
            End Get
            Set(ByVal value As StringAlignment)
                y1 = value
                Me.Refresh()
            End Set
        End Property


        Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
            MyBase.OnPaint(e)
            Dim rec As New Rectangle(0, 0, Me.Width, Me.Height)
            If Me.Width < 1 Then
                rec.Width = 1
                rec.Height = 1
            End If
            Dim blend As New ColorBlend
            Dim border As New Rectangle(BorderSize, BorderSize, Me.Width - BorderSize * 2, Me.Height - BorderSize * 2)
            Dim format As New StringFormat
            format.LineAlignment = TextAlinment_horizantal
            format.Alignment = TextAlinment_vertical
            Dim g As Graphics = e.Graphics
            Dim b As New LinearGradientBrush(rec, Me.BackColor, Me.BackColor, s.GradientAngle)
            Dim tex As New SolidBrush(ForeColor)
            Try
                If s.GradientColorArray.Count >= 2 And s.GradientPositions.Count >= 2 Then
                    blend.Colors = s.GradientColorArray.ToArray
                    blend.Positions = s.GradientPositions.ToArray
                    b.InterpolationColors = blend
                End If
                g.FillRectangle(b, rec)
                g.DrawString(Me.Text, Me.Font, tex, rec, format)
                g.ExcludeClip(border)
                If UseGradientBorderStyle Then
                    g.FillRectangle(borderStyle.ToLinearGradientBrush(e.ClipRectangle), e.ClipRectangle)
                Else
                    g.FillRectangle(New SolidBrush(BorderColor), e.ClipRectangle)
                End If
            Catch ex As Exception
            End Try
        End Sub


        Private Sub gradientBase_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Resize
            Me.Refresh()
        End Sub

        Private Sub gradientBase_TextChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.TextChanged
            Me.Refresh()
        End Sub

    End Class
End Namespace