﻿Imports System.ComponentModel
Imports System.ComponentModel.Design


Namespace GradientControls.styles
    <System.ComponentModel.DefaultProperty("GradientColorArray"), _
    System.ComponentModel.EditorBrowsable(System.ComponentModel.EditorBrowsableState.Always), _
    System.ComponentModel.ImmutableObject(False), _
    Description("This is the base tye that all style properties come from.")> _
    Public Class style

        Private p As New Collections.Generic.List(Of Single)(0)
        Private ang As Integer
        Private colo As New Collections.Generic.List(Of Color)(0)
        Private Const z As Single = 0

        Private Function validate(ByVal colors As List(Of Color)) As Boolean
            Dim good As Boolean
            good = Not colors Is Nothing
            Return good
        End Function


        ''' <summary>
        ''' Gets the ColorBlend of the style.
        ''' </summary>
        ''' <remarks>A ColorBlend contains the angle, colors, and positions, but alows for any size shape to be filled.</remarks>
        ''' <returns>ColorBlend</returns>
        Public Function ToColorBlend() As System.Drawing.Drawing2D.ColorBlend
            Dim temp As New System.Drawing.Drawing2D.ColorBlend
            Try
                temp.Colors = Me.GradientColorArray.ToArray
                temp.Positions = Me.GradientPositions.ToArray
            Catch ex As Exception
                Throw ex
            End Try
            Return temp
        End Function

        ''' <summary>
        ''' Gets the LinearGradientBrush of this style
        ''' </summary>
        ''' <param name="rect">The rectangle that the brush will be prepared to fill</param>
        ''' <remarks>A LinearGradientBrush has all angle, color, position, and drawing information for a definite size.</remarks>
        ''' <returns>LinearGradientBrush</returns>
        Public Function ToLinearGradientBrush(ByVal rect As Rectangle) As System.Drawing.Drawing2D.LinearGradientBrush
            Dim temp As New System.Drawing.Drawing2D.ColorBlend
            Dim a As New System.Drawing.Drawing2D.LinearGradientBrush(rect, Color.Black, Color.Black, GradientAngle)
            Try
                temp.Colors = Me.GradientColorArray.ToArray
                temp.Positions = Me.GradientPositions.ToArray
                a.InterpolationColors = temp
            Catch ex As Exception
                Throw ex
            End Try
            Return a
        End Function

        Private Function validate(ByVal positions As List(Of Single)) As Boolean
            Dim good As Boolean = False

            If positions Is Nothing Then
                good = False
            ElseIf positions.Count = 0 Then
                good = True
            ElseIf positions.Max <= 1 And positions.Min >= 0 Then
                good = True
            Else
                good = False
            End If

            Return good
        End Function
        Private Function validate(ByVal angle As Integer) As Boolean
            Dim good As Boolean
            good = -1 < angle And angle < 256


            Return good
        End Function

        <System.ComponentModel.Description("The array of positions as a decimal, E.G. 0.75, that corresponds to the gradient color of the same index in the multicolor gradient."), _
        System.ComponentModel.Category("Appearance")> _
        Public Property GradientPositions() As Collections.Generic.List(Of Single)
            Get
                Return p
            End Get
            Set(ByVal value As Collections.Generic.List(Of Single))
                If validate(value) Then
                    p = value
                End If

            End Set
        End Property

        <System.ComponentModel.Description("A generic list of colors that make up the multicolor gradient."), _
        System.ComponentModel.Category("Appearance")> _
        Public Property GradientColorArray() As Collections.Generic.List(Of Color)
            Get
                Return colo
            End Get
            Set(ByVal value As Collections.Generic.List(Of Color))
                If validate(value) Then
                    colo = value
                End If
            End Set
        End Property

        <System.ComponentModel.Description("The angle to rotate the gradient."), _
        System.ComponentModel.DefaultValue(90), _
        System.ComponentModel.Category("Appearance")> _
        Public Property GradientAngle() As Integer
            Get
                Return ang
            End Get
            Set(ByVal value As Integer)
                If validate(value) Then
                    ang = value
                Else
                    ang = 90
                End If
            End Set
        End Property

        Public Sub New(ByVal GradientColor As List(Of Color), ByVal GradientPosition As List(Of Single), Optional ByVal angle As Integer = 90)
            Me.GradientColorArray = GradientColor
            Me.GradientPositions = GradientPosition
            Me.GradientAngle = angle
        End Sub

        Public Sub New()
            Me.GradientColorArray = New List(Of Color)(0)
            Me.GradientPositions = New List(Of Single)(0)
            Me.GradientAngle = 90
        End Sub

    End Class
End Namespace
